-- listing 1.1 preparing tables
-- 1
CREATE SCHEMA ch01;
-- 2
CREATE TABLE ch01.lu_franchises(
			franchise_code char(1) PRIMARY KEY, 
			franchise_name varchar(100));
-- 3
INSERT INTO ch01.lu_franchises(franchise_code, franchise_name)
VALUES ('b', 'Burger King'),
	('c', 'Carl''s Jr'),
	('h', 'Hardee''s'),
	('i', 'In-N-Out'),
	('j', 'Jack in the Box'),
	('k', 'Kentucky Fried Chicken'),
	('m', 'McDonald''s'),
	('p', 'Pizza Hut'),
	('t', 'Taco Bell'),
	('w', 'Wendy''s');
-- 4
CREATE TABLE ch01.fastfoods
(
  franchise char(1) NOT NULL,
  lat double precision,
  lon double precision
);
-- 1 create logical container
-- 2 lookup table
-- 3 add franchise types
-- 4 table holds locations

-- Listing 1.2 Using geography data type to store data
-- 1
SELECT AddGeometryColumn('ch01', 'fastfoods', 'geom', 2163, 'POINT',2);
-- 2
UPDATE ch01.fastfoods 
	SET geom = 
		ST_Transform(ST_GeomFromText('POINT(' || lon || ' ' || lat || ')',4326), 2163);

-- 3
CREATE INDEX idx_fastfoods_geom ON ch01.fastfoods USING gist(geom);
vacuum analyze ch01.fastfoods;
-- 1 add column 
-- 2 use equal area planar
-- 3 general maintenance


-- Listing 1.3 Using geography data type to store data
-- 1
ALTER TABLE ch01.fastfoods ADD COLUMN geog geography(POINT,4326);
-- 2
UPDATE ch01.fastfoods 
	SET geog = 
		ST_GeogFromText('SRID=4326;POINT(' || lon || ' ' || lat || ')');
-- 3
vacuum analyze ch01.fastfoods;


-- foreign key and primary key
ALTER TABLE ch01.fastfoods ADD COLUMN ff_id SERIAL PRIMARY KEY;

ALTER TABLE ch01.fastfoods 
	ADD CONSTRAINT fk_fastfoods_franchise 
	FOREIGN KEY (franchise) 
	REFERENCES ch01.lu_franchise (franchise_code)
   ON UPDATE CASCADE ON DELETE RESTRICT;



-- Listing 1.4 Using geometry data type to store roads data
-- 1
SELECT AddGeometryColumn('ch01', 'roads', 'geom', 2163, 'MULTILINESTRING',2);
-- 2
UPDATE ch01.roads 
	SET geom = 
		ST_Transform(geom_4269, 2163);
-- 3
SELECT DropGeometryColumn('ch01', 'roads', 'geom_4269');

-- 4
CREATE INDEX idx_roads_geom ON ch01.roads USING gist(geom);
vacuum analyze ch01.roads;
-- 1 add column 
-- 2 use equal area planar meters
-- 3 drop old column
-- 4 general maintenance

--Listing 1.5 List franchise name, count of restaurants on a Principal Highway
SELECT ft.franchise_name, 
    -- 1
    COUNT(DISTINCT ff.ff_id) As tot
FROM ch01.fastfoods As ff 
    INNER JOIN ch01.lu_franchises As ft ON ff.franchise = ft.franchise_code
	INNER JOIN ch01.roads As r
	-- 2
	ON ST_DWithin(ff.geom, r.geom, 1609*1)
WHERE r.feature LIKE 'Principal Highway%' 
GROUP BY ft.franchise_name
-- 3
ORDER BY tot DESC;
-- 1 distinct count
-- 2 within 1 mile
-- 3 one with greatest total


--Listing 1.6 Return the prinicipal highway that has the most restaurants and how many
SELECT r.name,
-- 1
  COUNT(DISTINCT ff.ff_id) As tot
FROM ch01.fastfoods As ff 
	INNER JOIN ch01.roads As r 
-- 2
	ON ST_DWithin(ff.geom, r.geom, 1609*0.5)
WHERE r.feature LIKE 'Principal Highway%' 
GROUP BY r.name
-- 3
ORDER BY tot DESC LIMIT 1;
-- 1 distinct count
-- 2 within 1/2 mile
-- 3 one with greatest total